<?php
/**
 * BSS Commerce Co.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at thisURL:
 * http://bsscommerce.com/Bss-Commerce-License.txt
 *
 * @category   BSS
 * @package    Bss_AutoInvoice
 * @author     Extension Team
 * @copyright  Copyright (c) 2017-2018 BSS Commerce Co. ( http://bsscommerce.com )
 * @license    http://bsscommerce.com/Bss-Commerce-License.txt
 */
namespace Bss\AutoInvoice\Helper;

use Magento\Framework\App\Helper\Context;
use \Psr\Log\LoggerInterface;
use \Magento\Sales\Model\Order;

/**
 * Class Data
 *
 * @package Bss\AutoInvoice\Helper
 */
class Data extends \Magento\Framework\App\Helper\AbstractHelper
{

    /**
     * ProductMetadataInterface
     *
     * @var \Magento\Framework\App\ProductMetadataInterface $productMetadata
     */
    protected $productMetadata;

    /**
     * @var \Magento\Framework\DB\TransactionFactory
     */
    protected $transaction;

    /**
     * Data constructor.
     * @param Context $context
     * @param \Magento\Framework\App\ProductMetadataInterface $productMetadata
     * @param \Magento\Framework\DB\TransactionFactory $transaction
     */
    public function __construct(
        Context $context,
        \Magento\Framework\App\ProductMetadataInterface $productMetadata,
        \Magento\Framework\DB\TransactionFactory $transaction
    ) {
        $this->productMetadata = $productMetadata;
        $this->transaction = $transaction;
        parent::__construct($context);
    }

    /**
     * @return \Magento\Framework\DB\TransactionFactory
     */
    public function returnTransaction()
    {
        return $this->transaction;
    }

    /**
     * Check Enabled
     *
     * @return bool
     */
    public function isEnabled()
    {
        if ($this->isConfigActive() && ($this->isConfiginvoice() || $this->enabledShipment())) {
            return true;
        }
        return false;
    }

    /**
     * Check Payment methods
     *
     * @param $payment
     * @return bool
     */
    public function checkPaymentmethod($payment)
    {
        if (in_array($payment, explode(',', $this->getPaymentMethods()))) {
            return true;
        }
        return false;
    }

    /**
     * Is enabled auto create shipment
     *
     * @return bool
     */
    public function enabledShipment()
    {
        return $this->scopeConfig->isSetFlag(
            'autoinvoice/settings/shipment',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * Get Payment Methods
     *
     * @return string
     */
    public function getPaymentMethods()
    {
        return $this->scopeConfig->getValue(
            'autoinvoice/settings/payment_methods',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * Get Config active
     *
     * @return bool
     */
    public function isConfigActive()
    {
        return $this->scopeConfig->isSetFlag(
            'autoinvoice/settings/active',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * Get Config invoice
     *
     * @return bool
     */
    public function isConfiginvoice()
    {
        return $this->scopeConfig->isSetFlag(
            'autoinvoice/settings/invoice',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * Get Config select capture
     *
     * @return string
     */
    public function getSelectCapture()
    {
        return $this->scopeConfig->getValue(
            'autoinvoice/settings/capture',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * Get Magento Version
     *
     * @return bool
     */
    public function checkMagentoVersion()
    {
        $dataVersion = $this->productMetadata->getVersion();
        if (version_compare($dataVersion, '2.2.0') >= 0) {
            return false;
        }
        return true;
    }

    /**
     * @return LoggerInterface
     */
    public function returnLogger()
    {
        return $this->_logger;
    }

    /**
     * @param Order $order
     * @return bool
     */
    public function checkStateOrder($order)
    {
        return ($order->getState() == Order::STATE_NEW || $order->getState() == Order::STATE_PROCESSING);
    }
}
