<?php
/**
 * BSS Commerce Co.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at thisURL:
 * http://bsscommerce.com/Bss-Commerce-License.txt
 *
 * @category   BSS
 * @package    Bss_AutoInvoice
 * @author     Extension Team
 * @copyright  Copyright (c) 2017-2018 BSS Commerce Co. ( http://bsscommerce.com )
 * @license    http://bsscommerce.com/Bss-Commerce-License.txt
 */

namespace Bss\AutoInvoice\Model\Observer;

use Magento\Framework\Event\ObserverInterface;
use Magento\Sales\Model\Order;
use Bss\AutoInvoice\Helper\Data;
use Bss\AutoInvoice\Helper\AutoInvoice as AutoInvoiceHelper;

/**
 * Class AutoInvoice
 * @package Bss\AutoInvoice\Model\Observer
 */
class AutoInvoice implements ObserverInterface
{
    /**
     * Helper Data
     *
     * @var Data $helper
     */
    protected $helper;

    /**
     * @var AutoInvoiceHelper
     */
    protected $autoInvoiceHelper;

    /**
     * AutoInvoice constructor.
     * @param Data $helper
     * @param AutoInvoiceHelper $autoInvoiceHelper
     */
    public function __construct(
        Data $helper,
        AutoInvoiceHelper $autoInvoiceHelper
    ) {
        $this->helper = $helper;
        $this->autoInvoiceHelper = $autoInvoiceHelper;
    }

    /**
     * @param \Magento\Framework\Event\Observer $observer
     * @return bool|void
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        try {
            if ($this->helper->isEnabled()) {
                /** @var Order $order */
                $order = $observer->getEvent()->getOrder();
                if (!$order->getId()) {
                    throw new \Magento\Framework\Exception\LocalizedException(__('The order is no longer exists.'));
                }

                // Create invoice and shipment
                if (!$this->helper->checkPaymentmethod($order->getPayment()->getMethodInstance()->getCode())) {
                    return;
                }

                $this->doAutoInvoice($order);
            }
        } catch (\Exception $e) {
            $this->helper->returnLogger()->error($e->getMessage());
        }
    }

    /**
     * @param Order $order
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    protected function doAutoInvoice($order)
    {
        $paymentCode = $order->getPayment()->getMethodInstance()->getCode();
        switch ($paymentCode):
            case \Magento\Paypal\Model\Config::METHOD_EXPRESS:
                break;

            default:
                if ($this->helper->checkStateOrder($order)) {
                    if ($this->helper->isConfiginvoice() && $order->canInvoice()) {
                        $this->autoInvoiceHelper->createInvoice($order);
                    }

                    if ($this->helper->enabledShipment()) {
                        $this->autoInvoiceHelper->createShipment($order);
                    }
                }

                break;

        endswitch;
    }
}
